package luckdraw

import (
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"time"

	"github.com/go-xorm/xorm"
)

const (
	StatusNormal = 1
)

// DAO 当前数据库操作对象
type DAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewDAO 初始化DAO
func NewDAO(ctx *utils.Context) *DAO {
	return &DAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

const (
	STATUS_NOWRITROFF = 0
)

// LuckDraw 抽奖记录
type LuckDraw struct {
	model.TaLuckdraw `xorm:"extends"`
	Prizes           []model.TaLuckdrawPrize
}

// GetLuckDraw 获取抽奖信息
func (m *DAO) GetLuckDraw(id int) (*LuckDraw, error) {
	if id == 0 {
		return nil, utils.LogError("不存在抽奖信息！")
	}

	luckdraw, err := m.GetLuckDrawByID(id)
	if err != nil {
		return nil, err
	}
	prizes, err := m.GetPrizes(id)
	if err != nil {
		return nil, err
	}
	luckdraw.Prizes = prizes
	return luckdraw, nil
}

// LuckDraw 抽奖
func (m *DAO) LuckDraw(id int, user *model.TaCustomer) (*model.TaLuckdrawPrize, *model.TaPrizeDetail, error) {
	luckdraw, err := m.GetLuckDrawByID(id)
	if err != nil {
		return nil, nil, utils.LogError("获取抽奖信息失败！", err)
	}
	if luckdraw.Status != models.STATUS_NORMAL {
		return nil, nil, utils.LogError("当前抽奖活动状态异常！")
	}
	if time.Now().Before(luckdraw.BeginDate) {
		return nil, nil, utils.LogError("活动还未开始！")
	}
	if time.Now().After(luckdraw.EndDate) {
		return nil, nil, utils.LogError("活动已结束！")
	}
	if user.Phone != "" {
		sysuser, err := m.GetUserByTel(user.Phone)
		if err != nil {
			return nil, nil, utils.LogError("获取信息失败！", err)
		}
		if sysuser != nil {
			return nil, nil, utils.LogError("内部人员不允许进行抽奖！")
		}
	}
	userluckdraws, err := m.GetUserRecordByLuckDraw(user.CustomerId, id)
	if err != nil {
		return nil, nil, utils.LogError("获取用户抽奖信息失败！", err)
	}
	if len(userluckdraws) > 0 {
		return nil, nil, utils.LogError("您已参与过此次抽奖，不允许重复抽奖！")
	}
	prizes, err := m.GetPrizeStock(id)
	if err != nil {
		return nil, nil, utils.LogError("获取奖项失败！", err)
	}
	if len(prizes) == 0 {
		err = m.UpdateLuckDrawEndDate(luckdraw.Id)
		if err != nil {
			return nil, nil, utils.LogError("操作失败，请刷新后重试！", err)
		}
		return nil, nil, utils.LogError("活动已结束！")
	}
	prize, err := m.GetWinning(prizes)
	if err != nil {
		return nil, nil, utils.LogError("获取奖项失败！", err)
	}

	details, err := m.GetPrizeDetail(prize.Id)
	if err != nil {
		return nil, nil, utils.LogError("获取奖项详情失败！", err)
	}

	// 保存中奖记录
	var record = model.TaLuckdrawRecord{}
	record.OrgId = luckdraw.OrgId
	record.CaseId = luckdraw.CaseId
	record.PrizeId = prize.Id
	record.PrizeName = prize.PrizeName
	record.UserId = user.CustomerId
	record.UserName = user.Name
	record.UserHeadImg = user.Headimgurl
	record.LuckdrawId = luckdraw.Id
	newrecord, err := m.SaveRecord(record)
	var detail = model.TaPrizeDetail{}
	if len(details) > 0 {
		// 更新中奖纪录明细
		details[0].ReceiveId = newrecord.Id
		err = m.UpdatePrizeDetail(details[0])
		if err != nil {
			return nil, nil, utils.LogError("操作失败！", err)
		}
		detail = details[0]
		// 奖项自动核销，走卡券核销流程
		err = m.PrizeWriteOff(newrecord.Id, user.CustomerId)
		if err != nil {
			return nil, nil, utils.LogError("操作失败！", err)
		}
	}
	// 更新库存
	if prize.IsReality == 1 {
		prize.Remainder = prize.Remainder - 1
		err = m.UpdateStock(prize)
		if err != nil {
			return nil, nil, err
		}
		stockprizes, err := m.GetPrizeStock(id)
		if err != nil {
			return nil, nil, utils.LogError("操作失败，请刷新后重试！", err)
		}
		if len(stockprizes) == 0 {
			// 更新活动截止时间
			err = m.UpdateLuckDrawEndDate(luckdraw.Id)
		}
	}

	return prize, &detail, err
}

// PrizeWriteOff 奖品核销
func (m *DAO) PrizeWriteOff(recordid int, userid string) error {
	record, err := m.GetRecordByID(recordid)
	if err != nil {
		return err
	}
	var writeoff = model.TaLuckdrawWriteoff{}
	writeoff.LuckdrawId = record.LuckdrawId
	writeoff.CaseId = record.CaseId
	writeoff.OrgId = record.OrgId
	writeoff.RecordId = record.Id
	writeoff.WriteoffUser = userid
	_, err = m.SaveWirteOff(writeoff)
	if err != nil {
		return err
	}
	err = m.UpdateRecord(record)
	return err
}

// GetWinning 获取中奖信息
func (m *DAO) GetWinning(prizes []model.TaLuckdrawPrize) (*model.TaLuckdrawPrize, error) {
	pList := make([]map[string]interface{}, 0)
	now := time.Now().Local()

	for _, prize := range prizes {
		// 有剩余的 并且 未过期的
		if prize.Remainder > 0 && now.Before(prize.VerificationEnd) {
			p := map[string]interface{}{
				"prize": prize,
				"prob":  prize.Probability,
			}

			pList = append(pList, p)
		}
	}

	if len(pList) == 0 {
		return nil, utils.LogError("所有奖品已抽完")
	}

	thePrize := utils.LuckyDraw(pList).(model.TaLuckdrawPrize)

	return &thePrize, nil
}

// GetLuckDrawByID 获取抽奖信息
func (m *DAO) GetLuckDrawByID(id int) (*LuckDraw, error) {
	var luckdraws []LuckDraw
	sql := `select * from ta_luckdraw where id= ?`
	err := m.db.SQL(sql, id).Find(&luckdraws)
	if err != nil {
		return nil, err
	}
	if len(luckdraws) > 0 {
		return &luckdraws[0], nil
	}
	return nil, nil
}

// GetCaseAddressByLuckDarw 获取案场地址
func (m *DAO) GetCaseAddressByLuckDarw(luckdraw int) (string, error) {
	sql := `
		SELECT
			t.address
		FROM
			sys_case t
		INNER JOIN ta_luckdraw s ON t.case_id = s.case_id
		WHERE
			s.id = ?
	`

	res, err := m.db.Query(sql, luckdraw)
	if err != nil {
		return "", err
	}

	if res == nil || len(res) == 0 {
		return "", nil
	}

	return string(res[0]["address"]), nil
}

// UpdateLuckDrawEndDate 更新活动结束时间
func (m *DAO) UpdateLuckDrawEndDate(id int) error {
	var luckdraw = model.TaLuckdraw{
		Id:      id,
		EndDate: time.Now(),
	}
	cols := []string{
		"end_date",
	}
	_, err := m.db.Update(luckdraw, cols, "id=?", id)
	return err
}

// GetPrizes 获取奖品信息
func (m *DAO) GetPrizes(luckdrawid int) ([]model.TaLuckdrawPrize, error) {
	var prizes []model.TaLuckdrawPrize
	err := m.db.Where("luckdraw_id=?", luckdrawid).And("status=?", models.STATUS_NORMAL).Find(&prizes)
	return prizes, err
}

// UpdateStock 修改库存
func (m *DAO) UpdateStock(prize *model.TaLuckdrawPrize) error {
	cols := []string{
		"remainder",
	}
	_, err := m.db.Update(prize, cols, "id=?", prize.Id)
	return err
}

// GetPrizeStock 判断奖项库存
func (m *DAO) GetPrizeStock(luckdrawid int) ([]model.TaLuckdrawPrize, error) {
	var prizes []model.TaLuckdrawPrize
	err := m.db.Where("luckdraw_id=?", luckdrawid).And("status=?", models.STATUS_NORMAL).And("(remainder>0 or is_reality=0)").Find(&prizes)
	return prizes, err
}

// GetPrizeByID 根据id获取奖品信息
func (m *DAO) GetPrizeByID(id int) (model.TaLuckdrawPrize, error) {
	var prize = model.TaLuckdrawPrize{}
	_, err := m.db.Where("id=?", id).Get(&prize)
	return prize, err
}

// SaveRecord 保存抽奖记录
func (m *DAO) SaveRecord(record model.TaLuckdrawRecord) (*model.TaLuckdrawRecord, error) {
	record.CreateDate = time.Now()
	record.Status = STATUS_NOWRITROFF
	_, err := m.db.Insert(&record)
	return &record, err
}

// GetPrizeDetail 获取未领取的奖品详情
func (m *DAO) GetPrizeDetail(prizeid int) ([]model.TaPrizeDetail, error) {
	var details []model.TaPrizeDetail
	err := m.db.Where("prize_id=?", prizeid).And("status=?", STATUS_NOWRITROFF).Find(&details)
	return details, err
}

// UpdatePrizeDetail 更新库存明细为已领取
func (m *DAO) UpdatePrizeDetail(detail model.TaPrizeDetail) error {
	detail.Status = models.STATUS_NORMAL
	detail.ReceiveDate = time.Now()
	cols := []string{
		"status",
		"receive_id",
		"receive_date",
	}
	_, err := m.db.Update(detail, cols, "id=?", detail.Id)
	return err
}

// UpdateRecord 核销更新抽奖记录表
func (m *DAO) UpdateRecord(record *model.TaLuckdrawRecord) error {
	record.Status = models.STATUS_NORMAL
	record.WriteoffDate = time.Now()
	cols := []string{
		"status",
		"writeoff_date",
	}
	_, err := m.db.Update(record, cols, "id=?", record.Id)
	return err
}

// SaveWirteOff 保存核销信息
func (m *DAO) SaveWirteOff(writeoff model.TaLuckdrawWriteoff) (*model.TaLuckdrawWriteoff, error) {
	writeoff.Status = models.STATUS_NORMAL
	writeoff.WriteoffDate = time.Now()
	_, err := m.db.Insert(&writeoff)
	return &writeoff, err
}

// GetUserRecordByLuckDraw 获取用户的抽奖信息
func (m *DAO) GetUserRecordByLuckDraw(userid string, luckdrawid int) ([]model.TaLuckdrawRecord, error) {
	var records []model.TaLuckdrawRecord
	err := m.db.Where("user_id=?", userid).And("luckdraw_id=?", luckdrawid).And("status>" + strconv.Itoa(models.STATUS_DEL)).Find(&records)
	return records, err
}

// GetUserLuckDrawByLuckDraw 获取用户的抽奖信息
func (m *DAO) GetUserLuckDrawByLuckDraw(userid string, luckdrawid int) (*model.TaLuckdrawRecord, error) {
	var records []model.TaLuckdrawRecord
	err := m.db.Where("user_id=?", userid).And("luckdraw_id=?", luckdrawid).And("status>" + strconv.Itoa(models.STATUS_DEL)).Find(&records)
	if err != nil {
		return nil, err
	}
	if len(records) > 0 {
		return &records[0], nil
	}
	return nil, nil
}

// GetDetailByRecord 获取detail
func (m *DAO) GetDetailByRecord(recordid int) (*model.TaPrizeDetail, error) {
	var details []model.TaPrizeDetail
	err := m.db.Where("receive_id=?", recordid).And("status>" + strconv.Itoa(models.STATUS_DEL)).Find(&details)
	if err != nil {
		return nil, err
	}
	if len(details) > 0 {
		return &details[0], nil
	}
	return nil, nil
}

// UserRecord 用户抽奖信息
type UserRecord struct {
	model.TaLuckdrawRecord `xorm:"extends"`
	Url                    string
}

// GetUserLuckDraw 获取用户的抽奖信息
func (m *DAO) GetUserLuckDraw(userid string) ([]UserRecord, error) {
	var records []UserRecord
	sql := `select a.*,b.url from ta_luckdraw_record a left join ta_prize_detail b on a.id=b.receive_id where a.user_id=? and a.status>` + strconv.Itoa(models.STATUS_DEL)
	err := m.db.Sql(sql, userid).Find(&records)
	return records, err
}

// GetRecordByLuckDraw 抽奖记录
func (m *DAO) GetRecordByLuckDraw(luckdrawid int) ([]model.TaLuckdrawRecord, error) {
	var records []model.TaLuckdrawRecord
	err := m.db.Where("luckdraw_id=?", luckdrawid).And("status>" + strconv.Itoa(models.STATUS_DEL)).Desc("create_date").Find(&records)
	return records, err
}

// GetRecordByID 获取抽奖记录明细
func (m *DAO) GetRecordByID(id int) (*model.TaLuckdrawRecord, error) {
	var record model.TaLuckdrawRecord
	_, err := m.db.Where("id=?", id).Get(&record)
	return &record, err
}

// GetUserByOpenID 根据 openid 获取用户人员信息
func (m *DAO) GetUserByOpenID(openid string) (*model.SysUser, error) {
	var users []model.SysUser
	err := m.db.Where("open_id=?", openid).Find(&users)
	if err != nil {
		return nil, err
	}
	if len(users) > 0 {
		return &users[0], nil
	}
	return nil, nil
}

// GetUserByTel 根据 Tel 获取用户人员信息
func (m *DAO) GetUserByTel(tel string) (*model.SysUser, error) {
	var users = model.SysUser{}
	has, err := m.db.Where("phone=?", tel).Get(&users)
	if err != nil {
		return nil, err
	}
	if !has {
		return nil, nil
	}
	return &users, nil
}

// SaveLuckDrawShareData 保存用户分享信息
func (m *DAO) SaveLuckDrawShareData(dt *model.TaShareLuckyRecord) error {
	dt.CreateDate = time.Now().Local()
	dt.Status = StatusNormal

	if _, err := m.db.Insert(dt); err != nil {
		return err
	}

	return nil
}

// GetLuckDrawShareData 获取用户分享内容
func (m *DAO) GetLuckDrawShareData(luckID int, from, to, caseID string) ([]model.TaShareLuckyRecord, error) {
	var dts []model.TaShareLuckyRecord

	if err := m.db.Where("from_customer_id=?", from).
		And("to_customer_id=?", to).
		And("luckydraw_id=?", luckID).
		And("case_id=?", caseID).
		And("status=?", StatusNormal).
		Find(&dts); err != nil {
		return nil, err
	}

	return dts, nil
}
